from __future__ import division, absolute_import, print_function

__all__ = ['oldtype2dtype', 'convtypecode', 'convtypecode2', 'oldtypecodes']

import numpy as np

oldtype2dtype = {'1': np.dtype(np.byte),
                 's': np.dtype(np.short),
#                 'i': np.dtype(np.intc),
#                 'l': np.dtype(int),
#                 'b': np.dtype(np.ubyte),
                 'w': np.dtype(np.ushort),
                 'u': np.dtype(np.uintc),
#                 'f': np.dtype(np.single),
#                 'd': np.dtype(float),
#                 'F': np.dtype(np.csingle),
#                 'D': np.dtype(complex),
#                 'O': np.dtype(object),
#                 'c': np.dtype('c'),
                 None: np.dtype(int)
    }

# converts typecode=None to int
def convtypecode(typecode, dtype=None):
    if dtype is None:
        try:
            return oldtype2dtype[typecode]
        except:
            return np.dtype(typecode)
    else:
        return dtype

#if both typecode and dtype are None
#  return None
def convtypecode2(typecode, dtype=None):
    if dtype is None:
        if typecode is None:
            return None
        else:
            try:
                return oldtype2dtype[typecode]
            except:
                return np.dtype(typecode)
    else:
        return dtype

_changedtypes = {'B': 'b',
                 'b': '1',
                 'h': 's',
                 'H': 'w',
                 'I': 'u'}

class _oldtypecodes(dict):
    def __getitem__(self, obj):
        char = np.dtype(obj).char
        try:
            return _changedtypes[char]
        except KeyError:
            return char


oldtypecodes = _oldtypecodes()



"""Here we define the exported functions, types, etc... which need to be
exported through a global C pointer.

Each dictionary contains name -> index pair.

Whenever you change one index, you break the ABI (and the ABI version number
should be incremented). Whenever you add an item to one of the dict, the API
needs to be updated.

When adding a function, make sure to use the next integer not used as an index
(in case you use an existing index or jump, the build will stop and raise an
exception, so it should hopefully not get unnoticed).

"""
from __future__ import division, absolute_import, print_function

multiarray_global_vars = {
    'NPY_NUMUSERTYPES':             7,
    'NPY_DEFAULT_ASSIGN_CASTING':   292,
}

multiarray_global_vars_types = {
    'NPY_NUMUSERTYPES':             'int',
    'NPY_DEFAULT_ASSIGN_CASTING':   'NPY_CASTING',
}

multiarray_scalar_bool_values = {
    '_PyArrayScalar_BoolValues':    9
}

multiarray_types_api = {
    'PyBigArray_Type':                  1,
    'PyArray_Type':                     2,
    'PyArrayDescr_Type':                3,
    'PyArrayFlags_Type':                4,
    'PyArrayIter_Type':                 5,
    'PyArrayMultiIter_Type':            6,
    'PyBoolArrType_Type':               8,
    'PyGenericArrType_Type':            10,
    'PyNumberArrType_Type':             11,
    'PyIntegerArrType_Type':            12,
    'PySignedIntegerArrType_Type':      13,
    'PyUnsignedIntegerArrType_Type':    14,
    'PyInexactArrType_Type':            15,
    'PyFloatingArrType_Type':           16,
    'PyComplexFloatingArrType_Type':    17,
    'PyFlexibleArrType_Type':           18,
    'PyCharacterArrType_Type':          19,
    'PyByteArrType_Type':               20,
    'PyShortArrType_Type':              21,
    'PyIntArrType_Type':                22,
    'PyLongArrType_Type':               23,
    'PyLongLongArrType_Type':           24,
    'PyUByteArrType_Type':              25,
    'PyUShortArrType_Type':             26,
    'PyUIntArrType_Type':               27,
    'PyULongArrType_Type':              28,
    'PyULongLongArrType_Type':          29,
    'PyFloatArrType_Type':              30,
    'PyDoubleArrType_Type':             31,
    'PyLongDoubleArrType_Type':         32,
    'PyCFloatArrType_Type':             33,
    'PyCDoubleArrType_Type':            34,
    'PyCLongDoubleArrType_Type':        35,
    'PyObjectArrType_Type':             36,
    'PyStringArrType_Type':             37,
    'PyUnicodeArrType_Type':            38,
    'PyVoidArrType_Type':               39,
    # End 1.5 API
    'PyTimeIntegerArrType_Type':        214,
    'PyDatetimeArrType_Type':           215,
    'PyTimedeltaArrType_Type':          216,
    'PyHalfArrType_Type':               217,
    'NpyIter_Type':                     218,
    # End 1.6 API
}

#define NPY_NUMUSERTYPES (*(int *)PyArray_API[6])
#define PyBoolArrType_Type (*(PyTypeObject *)PyArray_API[7])
#define _PyArrayScalar_BoolValues ((PyBoolScalarObject *)PyArray_API[8])

multiarray_funcs_api = {
    'PyArray_GetNDArrayCVersion':           0,
    'PyArray_SetNumericOps':                40,
    'PyArray_GetNumericOps':                41,
    'PyArray_INCREF':                       42,
    'PyArray_XDECREF':                      43,
    'PyArray_SetStringFunction':            44,
    'PyArray_DescrFromType':                45,
    'PyArray_TypeObjectFromType':           46,
    'PyArray_Zero':                         47,
    'PyArray_One':                          48,
    'PyArray_CastToType':                   49,
    'PyArray_CastTo':                       50,
    'PyArray_CastAnyTo':                    51,
    'PyArray_CanCastSafely':                52,
    'PyArray_CanCastTo':                    53,
    'PyArray_ObjectType':                   54,
    'PyArray_DescrFromObject':              55,
    'PyArray_ConvertToCommonType':          56,
    'PyArray_DescrFromScalar':              57,
    'PyArray_DescrFromTypeObject':          58,
    'PyArray_Size':                         59,
    'PyArray_Scalar':                       60,
    'PyArray_FromScalar':                   61,
    'PyArray_ScalarAsCtype':                62,
    'PyArray_CastScalarToCtype':            63,
    'PyArray_CastScalarDirect':             64,
    'PyArray_ScalarFromObject':             65,
    'PyArray_GetCastFunc':                  66,
    'PyArray_FromDims':                     67,
    'PyArray_FromDimsAndDataAndDescr':      68,
    'PyArray_FromAny':                      69,
    'PyArray_EnsureArray':                  70,
    'PyArray_EnsureAnyArray':               71,
    'PyArray_FromFile':                     72,
    'PyArray_FromString':                   73,
    'PyArray_FromBuffer':                   74,
    'PyArray_FromIter':                     75,
    'PyArray_Return':                       76,
    'PyArray_GetField':                     77,
    'PyArray_SetField':                     78,
    'PyArray_Byteswap':                     79,
    'PyArray_Resize':                       80,
    'PyArray_MoveInto':                     81,
    'PyArray_CopyInto':                     82,
    'PyArray_CopyAnyInto':                  83,
    'PyArray_CopyObject':                   84,
    'PyArray_NewCopy':                      85,
    'PyArray_ToList':                       86,
    'PyArray_ToString':                     87,
    'PyArray_ToFile':                       88,
    'PyArray_Dump':                         89,
    'PyArray_Dumps':                        90,
    'PyArray_ValidType':                    91,
    'PyArray_UpdateFlags':                  92,
    'PyArray_New':                          93,
    'PyArray_NewFromDescr':                 94,
    'PyArray_DescrNew':                     95,
    'PyArray_DescrNewFromType':             96,
    'PyArray_GetPriority':                  97,
    'PyArray_IterNew':                      98,
    'PyArray_MultiIterNew':                 99,
    'PyArray_PyIntAsInt':                   100,
    'PyArray_PyIntAsIntp':                  101,
    'PyArray_Broadcast':                    102,
    'PyArray_FillObjectArray':              103,
    'PyArray_FillWithScalar':               104,
    'PyArray_CheckStrides':                 105,
    'PyArray_DescrNewByteorder':            106,
    'PyArray_IterAllButAxis':               107,
    'PyArray_CheckFromAny':                 108,
    'PyArray_FromArray':                    109,
    'PyArray_FromInterface':                110,
    'PyArray_FromStructInterface':          111,
    'PyArray_FromArrayAttr':                112,
    'PyArray_ScalarKind':                   113,
    'PyArray_CanCoerceScalar':              114,
    'PyArray_NewFlagsObject':               115,
    'PyArray_CanCastScalar':                116,
    'PyArray_CompareUCS4':                  117,
    'PyArray_RemoveSmallest':               118,
    'PyArray_ElementStrides':               119,
    'PyArray_Item_INCREF':                  120,
    'PyArray_Item_XDECREF':                 121,
    'PyArray_FieldNames':                   122,
    'PyArray_Transpose':                    123,
    'PyArray_TakeFrom':                     124,
    'PyArray_PutTo':                        125,
    'PyArray_PutMask':                      126,
    'PyArray_Repeat':                       127,
    'PyArray_Choose':                       128,
    'PyArray_Sort':                         129,
    'PyArray_ArgSort':                      130,
    'PyArray_SearchSorted':                 131,
    'PyArray_ArgMax':                       132,
    'PyArray_ArgMin':                       133,
    'PyArray_Reshape':                      134,
    'PyArray_Newshape':                     135,
    'PyArray_Squeeze':                      136,
    'PyArray_View':                         137,
    'PyArray_SwapAxes':                     138,
    'PyArray_Max':                          139,
    'PyArray_Min':                          140,
    'PyArray_Ptp':                          141,
    'PyArray_Mean':                         142,
    'PyArray_Trace':                        143,
    'PyArray_Diagonal':                     144,
    'PyArray_Clip':                         145,
    'PyArray_Conjugate':                    146,
    'PyArray_Nonzero':                      147,
    'PyArray_Std':                          148,
    'PyArray_Sum':                          149,
    'PyArray_CumSum':                       150,
    'PyArray_Prod':                         151,
    'PyArray_CumProd':                      152,
    'PyArray_All':                          153,
    'PyArray_Any':                          154,
    'PyArray_Compress':                     155,
    'PyArray_Flatten':                      156,
    'PyArray_Ravel':                        157,
    'PyArray_MultiplyList':                 158,
    'PyArray_MultiplyIntList':              159,
    'PyArray_GetPtr':                       160,
    'PyArray_CompareLists':                 161,
    'PyArray_AsCArray':                     162,
    'PyArray_As1D':                         163,
    'PyArray_As2D':                         164,
    'PyArray_Free':                         165,
    'PyArray_Converter':                    166,
    'PyArray_IntpFromSequence':             167,
    'PyArray_Concatenate':                  168,
    'PyArray_InnerProduct':                 169,
    'PyArray_MatrixProduct':                170,
    'PyArray_CopyAndTranspose':             171,
    'PyArray_Correlate':                    172,
    'PyArray_TypestrConvert':               173,
    'PyArray_DescrConverter':               174,
    'PyArray_DescrConverter2':              175,
    'PyArray_IntpConverter':                176,
    'PyArray_BufferConverter':              177,
    'PyArray_AxisConverter':                178,
    'PyArray_BoolConverter':                179,
    'PyArray_ByteorderConverter':           180,
    'PyArray_OrderConverter':               181,
    'PyArray_EquivTypes':                   182,
    'PyArray_Zeros':                        183,
    'PyArray_Empty':                        184,
    'PyArray_Where':                        185,
    'PyArray_Arange':                       186,
    'PyArray_ArangeObj':                    187,
    'PyArray_SortkindConverter':            188,
    'PyArray_LexSort':                      189,
    'PyArray_Round':                        190,
    'PyArray_EquivTypenums':                191,
    'PyArray_RegisterDataType':             192,
    'PyArray_RegisterCastFunc':             193,
    'PyArray_RegisterCanCast':              194,
    'PyArray_InitArrFuncs':                 195,
    'PyArray_IntTupleFromIntp':             196,
    'PyArray_TypeNumFromName':              197,
    'PyArray_ClipmodeConverter':            198,
    'PyArray_OutputConverter':              199,
    'PyArray_BroadcastToShape':             200,
    '_PyArray_SigintHandler':               201,
    '_PyArray_GetSigintBuf':                202,
    'PyArray_DescrAlignConverter':          203,
    'PyArray_DescrAlignConverter2':         204,
    'PyArray_SearchsideConverter':          205,
    'PyArray_CheckAxis':                    206,
    'PyArray_OverflowMultiplyList':         207,
    'PyArray_CompareString':                208,
    'PyArray_MultiIterFromObjects':         209,
    'PyArray_GetEndianness':                210,
    'PyArray_GetNDArrayCFeatureVersion':    211,
    'PyArray_Correlate2':                   212,
    'PyArray_NeighborhoodIterNew':          213,
    # End 1.5 API
    'PyArray_SetDatetimeParseFunction':     219,
    'PyArray_DatetimeToDatetimeStruct':     220,
    'PyArray_TimedeltaToTimedeltaStruct':   221,
    'PyArray_DatetimeStructToDatetime':     222,
    'PyArray_TimedeltaStructToTimedelta':   223,
    # NDIter API
    'NpyIter_New':                          224,
    'NpyIter_MultiNew':                     225,
    'NpyIter_AdvancedNew':                  226,
    'NpyIter_Copy':                         227,
    'NpyIter_Deallocate':                   228,
    'NpyIter_HasDelayedBufAlloc':           229,
    'NpyIter_HasExternalLoop':              230,
    'NpyIter_EnableExternalLoop':           231,
    'NpyIter_GetInnerStrideArray':          232,
    'NpyIter_GetInnerLoopSizePtr':          233,
    'NpyIter_Reset':                        234,
    'NpyIter_ResetBasePointers':            235,
    'NpyIter_ResetToIterIndexRange':        236,
    'NpyIter_GetNDim':                      237,
    'NpyIter_GetNOp':                       238,
    'NpyIter_GetIterNext':                  239,
    'NpyIter_GetIterSize':                  240,
    'NpyIter_GetIterIndexRange':            241,
    'NpyIter_GetIterIndex':                 242,
    'NpyIter_GotoIterIndex':                243,
    'NpyIter_HasMultiIndex':                244,
    'NpyIter_GetShape':                     245,
    'NpyIter_GetGetMultiIndex':             246,
    'NpyIter_GotoMultiIndex':               247,
    'NpyIter_RemoveMultiIndex':             248,
    'NpyIter_HasIndex':                     249,
    'NpyIter_IsBuffered':                   250,
    'NpyIter_IsGrowInner':                  251,
    'NpyIter_GetBufferSize':                252,
    'NpyIter_GetIndexPtr':                  253,
    'NpyIter_GotoIndex':                    254,
    'NpyIter_GetDataPtrArray':              255,
    'NpyIter_GetDescrArray':                256,
    'NpyIter_GetOperandArray':              257,
    'NpyIter_GetIterView':                  258,
    'NpyIter_GetReadFlags':                 259,
    'NpyIter_GetWriteFlags':                260,
    'NpyIter_DebugPrint':                   261,
    'NpyIter_IterationNeedsAPI':            262,
    'NpyIter_GetInnerFixedStrideArray':     263,
    'NpyIter_RemoveAxis':                   264,
    'NpyIter_GetAxisStrideArray':           265,
    'NpyIter_RequiresBuffering':            266,
    'NpyIter_GetInitialDataPtrArray':       267,
    'NpyIter_CreateCompatibleStrides':      268,
    #
    'PyArray_CastingConverter':             269,
    'PyArray_CountNonzero':                 270,
    'PyArray_PromoteTypes':                 271,
    'PyArray_MinScalarType':                272,
    'PyArray_ResultType':                   273,
    'PyArray_CanCastArrayTo':               274,
    'PyArray_CanCastTypeTo':                275,
    'PyArray_EinsteinSum':                  276,
    'PyArray_NewLikeArray':                 277,
    'PyArray_GetArrayParamsFromObject':     278,
    'PyArray_ConvertClipmodeSequence':      279,
    'PyArray_MatrixProduct2':               280,
    # End 1.6 API
    'NpyIter_IsFirstVisit':                 281,
    'PyArray_SetBaseObject':                282,
    'PyArray_CreateSortedStridePerm':       283,
    'PyArray_RemoveAxesInPlace':            284,
    'PyArray_DebugPrint':                   285,
    'PyArray_FailUnlessWriteable':          286,
    'PyArray_SetUpdateIfCopyBase':          287,
    'PyDataMem_NEW':                        288,
    'PyDataMem_FREE':                       289,
    'PyDataMem_RENEW':                      290,
    'PyDataMem_SetEventHook':               291,
    'PyArray_MapIterSwapAxes':              293,
    'PyArray_MapIterArray':                 294,
    'PyArray_MapIterNext':                  295,
    # End 1.7 API
    'PyArray_Partition':                    296,
    'PyArray_ArgPartition':                 297,
    'PyArray_SelectkindConverter':          298,
    'PyDataMem_NEW_ZEROED':                 299,
    # End 1.8 API
}

ufunc_types_api = {
    'PyUFunc_Type':                             0
}

ufunc_funcs_api = {
    'PyUFunc_FromFuncAndData':                  1,
    'PyUFunc_RegisterLoopForType':              2,
    'PyUFunc_GenericFunction':                  3,
    'PyUFunc_f_f_As_d_d':                       4,
    'PyUFunc_d_d':                              5,
    'PyUFunc_f_f':                              6,
    'PyUFunc_g_g':                              7,
    'PyUFunc_F_F_As_D_D':                       8,
    'PyUFunc_F_F':                              9,
    'PyUFunc_D_D':                              10,
    'PyUFunc_G_G':                              11,
    'PyUFunc_O_O':                              12,
    'PyUFunc_ff_f_As_dd_d':                     13,
    'PyUFunc_ff_f':                             14,
    'PyUFunc_dd_d':                             15,
    'PyUFunc_gg_g':                             16,
    'PyUFunc_FF_F_As_DD_D':                     17,
    'PyUFunc_DD_D':                             18,
    'PyUFunc_FF_F':                             19,
    'PyUFunc_GG_G':                             20,
    'PyUFunc_OO_O':                             21,
    'PyUFunc_O_O_method':                       22,
    'PyUFunc_OO_O_method':                      23,
    'PyUFunc_On_Om':                            24,
    'PyUFunc_GetPyValues':                      25,
    'PyUFunc_checkfperr':                       26,
    'PyUFunc_clearfperr':                       27,
    'PyUFunc_getfperr':                         28,
    'PyUFunc_handlefperr':                      29,
    'PyUFunc_ReplaceLoopBySignature':           30,
    'PyUFunc_FromFuncAndDataAndSignature':      31,
    'PyUFunc_SetUsesArraysAsData':              32,
    # End 1.5 API
    'PyUFunc_e_e':                              33,
    'PyUFunc_e_e_As_f_f':                       34,
    'PyUFunc_e_e_As_d_d':                       35,
    'PyUFunc_ee_e':                             36,
    'PyUFunc_ee_e_As_ff_f':                     37,
    'PyUFunc_ee_e_As_dd_d':                     38,
    # End 1.6 API
    'PyUFunc_DefaultTypeResolver':              39,
    'PyUFunc_ValidateCasting':                  40,
    # End 1.7 API
    'PyUFunc_RegisterLoopForDescr':             41,
    # End 1.8 API
}

# List of all the dicts which define the C API
# XXX: DO NOT CHANGE THE ORDER OF TUPLES BELOW !
multiarray_api = (
        multiarray_global_vars,
        multiarray_global_vars_types,
        multiarray_scalar_bool_values,
        multiarray_types_api,
        multiarray_funcs_api,
)

ufunc_api = (
        ufunc_funcs_api,
        ufunc_types_api
)

full_api = multiarray_api + ufunc_api



"""
A buffered iterator for big arrays.

This module solves the problem of iterating over a big file-based array
without having to read it into memory. The `Arrayterator` class wraps
an array object, and when iterated it will return sub-arrays with at most
a user-specified number of elements.

"""
from __future__ import division, absolute_import, print_function

import sys
from operator import mul
from functools import reduce

from numpy.compat import long

__all__ = ['Arrayterator']


class Arrayterator(object):
    """
    Buffered iterator for big arrays.

    `Arrayterator` creates a buffered iterator for reading big arrays in small
    contiguous blocks. The class is useful for objects stored in the
    file system. It allows iteration over the object *without* reading
    everything in memory; instead, small blocks are read and iterated over.

    `Arrayterator` can be used with any object that supports multidimensional
    slices. This includes NumPy arrays, but also variables from
    Scientific.IO.NetCDF or pynetcdf for example.

    Parameters
    ----------
    var : array_like
        The object to iterate over.
    buf_size : int, optional
        The buffer size. If `buf_size` is supplied, the maximum amount of
        data that will be read into memory is `buf_size` elements.
        Default is None, which will read as many element as possible
        into memory.

    Attributes
    ----------
    var
    buf_size
    start
    stop
    step
    shape
    flat

    See Also
    --------
    ndenumerate : Multidimensional array iterator.
    flatiter : Flat array iterator.
    memmap : Create a memory-map to an array stored in a binary file on disk.

    Notes
    -----
    The algorithm works by first finding a "running dimension", along which
    the blocks will be extracted. Given an array of dimensions
    ``(d1, d2, ..., dn)``, e.g. if `buf_size` is smaller than ``d1``, the
    first dimension will be used. If, on the other hand,
    ``d1 < buf_size < d1*d2`` the second dimension will be used, and so on.
    Blocks are extracted along this dimension, and when the last block is
    returned the process continues from the next dimension, until all
    elements have been read.

    Examples
    --------
    >>> import numpy as np
    >>> a = np.arange(3 * 4 * 5 * 6).reshape(3, 4, 5, 6)
    >>> a_itor = np.lib.arrayterator.Arrayterator(a, 2)
    >>> a_itor.shape
    (3, 4, 5, 6)

    Now we can iterate over ``a_itor``, and it will return arrays of size
    two. Since `buf_size` was smaller than any dimension, the first
    dimension will be iterated over first:

    >>> for subarr in a_itor:
    ...     if not subarr.all():
    ...         print subarr, subarr.shape
    ...
    [[[[0 1]]]] (1, 1, 1, 2)

    """

    def __init__(self, var, buf_size=None):
        self.var = var
        self.buf_size = buf_size

        self.start = [0 for dim in var.shape]
        self.stop = [dim for dim in var.shape]
        self.step = [1 for dim in var.shape]

    def __getattr__(self, attr):
        return getattr(self.var, attr)

    def __getitem__(self, index):
        """
        Return a new arrayterator.

        """
        # Fix index, handling ellipsis and incomplete slices.
        if not isinstance(index, tuple): index = (index,)
        fixed = []
        length, dims = len(index), len(self.shape)
        for slice_ in index:
            if slice_ is Ellipsis:
                fixed.extend([slice(None)] * (dims-length+1))
                length = len(fixed)
            elif isinstance(slice_, (int, long)):
                fixed.append(slice(slice_, slice_+1, 1))
            else:
                fixed.append(slice_)
        index = tuple(fixed)
        if len(index) < dims:
            index += (slice(None),) * (dims-len(index))

        # Return a new arrayterator object.
        out = self.__class__(self.var, self.buf_size)
        for i, (start, stop, step, slice_) in enumerate(
                zip(self.start, self.stop, self.step, index)):
            out.start[i] = start + (slice_.start or 0)
            out.step[i] = step * (slice_.step or 1)
            out.stop[i] = start + (slice_.stop or stop-start)
            out.stop[i] = min(stop, out.stop[i])
        return out

    def __array__(self):
        """
        Return corresponding data.

        """
        slice_ = tuple(slice(*t) for t in zip(
                self.start, self.stop, self.step))
        return self.var[slice_]

    @property
    def flat(self):
        """
        A 1-D flat iterator for Arrayterator objects.

        This iterator returns elements of the array to be iterated over in
        `Arrayterator` one by one. It is similar to `flatiter`.

        See Also
        --------
        `Arrayterator`
        flatiter

        Examples
        --------
        >>> a = np.arange(3 * 4 * 5 * 6).reshape(3, 4, 5, 6)
        >>> a_itor = np.lib.arrayterator.Arrayterator(a, 2)

        >>> for subarr in a_itor.flat:
        ...     if not subarr:
        ...         print subarr, type(subarr)
        ...
        0 <type 'numpy.int32'>

        """
        for block in self:
            for value in block.flat:
                yield value

    @property
    def shape(self):
        """
        The shape of the array to be iterated over.

        For an example, see `Arrayterator`.

        """
        return tuple(((stop-start-1)//step+1) for start, stop, step in
                zip(self.start, self.stop, self.step))

    def __iter__(self):
        # Skip arrays with degenerate dimensions
        if [dim for dim in self.shape if dim <= 0]: raise StopIteration

        start = self.start[:]
        stop = self.stop[:]
        step = self.step[:]
        ndims = len(self.var.shape)

        while True:
            count = self.buf_size or reduce(mul, self.shape)

            # iterate over each dimension, looking for the
            # running dimension (ie, the dimension along which
            # the blocks will be built from)
            rundim = 0
            for i in range(ndims-1, -1, -1):
                # if count is zero we ran out of elements to read
                # along higher dimensions, so we read only a single position
                if count == 0:
                    stop[i] = start[i]+1
                elif count <= self.shape[i]:  # limit along this dimension
                    stop[i] = start[i] + count*step[i]
                    rundim = i
                else:
                    stop[i] = self.stop[i]  # read everything along this
                                            # dimension
                stop[i] = min(self.stop[i], stop[i])
                count = count//self.shape[i]

            # yield a block
            slice_ = tuple(slice(*t) for t in zip(start, stop, step))
            yield self.var[slice_]

            # Update start position, taking care of overflow to
            # other dimensions
            start[rundim] = stop[rundim]  # start where we stopped
            for i in range(ndims-1, 0, -1):
                if start[i] >= self.stop[i]:
                    start[i] = self.start[i]
                    start[i-1] += self.step[i-1]
            if start[0] >= self.stop[0]:
                raise StopIteration



# -*- Mode: Python -*-  Not really, but close enough
"""WARNING: this code is deprecated and slated for removal soon.  See the
doc/cython directory for the replacement, which uses Cython (the actively
maintained version of Pyrex).
"""

cimport c_python
cimport c_numpy
import numpy

# Numpy must be initialized
c_numpy.import_array()

def print_array_info(c_numpy.ndarray arr):
    cdef int i

    print '-='*10
    print 'printing array info for ndarray at 0x%0lx'%(<c_python.Py_intptr_t>arr,)
    print 'print number of dimensions:',arr.nd
    print 'address of strides: 0x%0lx'%(<c_python.Py_intptr_t>arr.strides,)
    print 'strides:'
    for i from 0<=i<arr.nd:
        # print each stride
        print '  stride %d:'%i,<c_python.Py_intptr_t>arr.strides[i]
    print 'memory dump:'
    print_elements( arr.data, arr.strides, arr.dimensions,
                    arr.nd, sizeof(double), arr.dtype )
    print '-='*10
    print

cdef print_elements(char *data,
                    c_python.Py_intptr_t* strides,
                    c_python.Py_intptr_t* dimensions,
                    int nd,
                    int elsize,
                    object dtype):
    cdef c_python.Py_intptr_t i,j
    cdef void* elptr

    if dtype not in [numpy.dtype(numpy.object_),
                     numpy.dtype(numpy.float64)]:
        print '   print_elements() not (yet) implemented for dtype %s'%dtype.name
        return

    if nd ==0:
        if dtype==numpy.dtype(numpy.object_):
            elptr = (<void**>data)[0] #[0] dereferences pointer in Pyrex
            print '  ',<object>elptr
        elif dtype==numpy.dtype(numpy.float64):
            print '  ',(<double*>data)[0]
    elif nd == 1:
        for i from 0<=i<dimensions[0]:
            if dtype==numpy.dtype(numpy.object_):
                elptr = (<void**>data)[0]
                print '  ',<object>elptr
            elif dtype==numpy.dtype(numpy.float64):
                print '  ',(<double*>data)[0]
            data = data + strides[0]
    else:
        for i from 0<=i<dimensions[0]:
            print_elements(data, strides+1, dimensions+1, nd-1, elsize, dtype)
            data = data + strides[0]

def test_methods(c_numpy.ndarray arr):
    """Test a few attribute accesses for an array.
    
    This illustrates how the pyrex-visible object is in practice a strange
    hybrid of the C PyArrayObject struct and the python object.  Some
    properties (like .nd) are visible here but not in python, while others
    like flags behave very differently: in python flags appears as a separate,
    object while here we see the raw int holding the bit pattern.

    This makes sense when we think of how pyrex resolves arr.foo: if foo is
    listed as a field in the c_numpy.ndarray struct description, it will be
    directly accessed as a C variable without going through Python at all.
    This is why for arr.flags, we see the actual int which holds all the flags
    as bit fields.  However, for any other attribute not listed in the struct,
    it simply forwards the attribute lookup to python at runtime, just like
    python would (which means that AttributeError can  be raised for
    non-existent attributes, for example)."""
    
    print 'arr.any() :',arr.any()
    print 'arr.nd    :',arr.nd
    print 'arr.flags :',arr.flags

def test():
    """this function is pure Python"""
    arr1 = numpy.array(-1e-30,dtype=numpy.float64)
    arr2 = numpy.array([1.0,2.0,3.0],dtype=numpy.float64)

    arr3 = numpy.arange(9,dtype=numpy.float64)
    arr3.shape = 3,3

    four = 4
    arr4 = numpy.array(['one','two',3,four],dtype=numpy.object_)

    arr5 = numpy.array([1,2,3]) # int types not (yet) supported by print_elements

    for arr in [arr1,arr2,arr3,arr4,arr5]:
        print_array_info(arr)




from __future__ import division, absolute_import, print_function

__all__ = ['average', 'histogram', 'standardDeviation', 'variance']

import numpy.oldnumeric as Numeric

def average(data):
    data = Numeric.array(data)
    return Numeric.add.reduce(data)/len(data)

def variance(data):
    data = Numeric.array(data)
    return Numeric.add.reduce((data-average(data, axis=0))**2)/(len(data)-1)

def standardDeviation(data):
    data = Numeric.array(data)
    return Numeric.sqrt(variance(data))

def histogram(data, nbins, range = None):
    data = Numeric.array(data, Numeric.Float)
    if range is None:
        min = Numeric.minimum.reduce(data)
        max = Numeric.maximum.reduce(data)
    else:
        min, max = range
        data = Numeric.repeat(data,
                              Numeric.logical_and(Numeric.less_equal(data, max),
                                                  Numeric.greater_equal(data,
                                                                        min)), axis=0)
    bin_width = (max-min)/nbins
    data = Numeric.floor((data - min)/bin_width).astype(Numeric.Int)
    histo = Numeric.add.reduce(Numeric.equal(
        Numeric.arange(nbins)[:, Numeric.NewAxis], data), -1)
    histo[-1] = histo[-1] + Numeric.add.reduce(Numeric.equal(nbins, data))
    bins = min + bin_width*(Numeric.arange(nbins)+0.5)
    return Numeric.transpose(Numeric.array([bins, histo]))



"""Functions that should behave the same as Numeric and need changing

"""
from __future__ import division, absolute_import, print_function

import numpy as np
import numpy.core.multiarray as mu
import numpy.core.numeric as nn
from .typeconv import convtypecode, convtypecode2

__all__ = ['take', 'repeat', 'sum', 'product', 'sometrue', 'alltrue',
           'cumsum', 'cumproduct', 'compress', 'fromfunction',
           'ones', 'empty', 'identity', 'zeros', 'array', 'asarray',
           'nonzero', 'reshape', 'arange', 'fromstring', 'ravel', 'trace',
           'indices', 'where', 'sarray', 'cross_product', 'argmax', 'argmin',
           'average']

def take(a, indicies, axis=0):
    return np.take(a, indicies, axis)

def repeat(a, repeats, axis=0):
    return np.repeat(a, repeats, axis)

def sum(x, axis=0):
    return np.sum(x, axis)

def product(x, axis=0):
    return np.product(x, axis)

def sometrue(x, axis=0):
    return np.sometrue(x, axis)

def alltrue(x, axis=0):
    return np.alltrue(x, axis)

def cumsum(x, axis=0):
    return np.cumsum(x, axis)

def cumproduct(x, axis=0):
    return np.cumproduct(x, axis)

def argmax(x, axis=-1):
    return np.argmax(x, axis)

def argmin(x, axis=-1):
    return np.argmin(x, axis)

def compress(condition, m, axis=-1):
    return np.compress(condition, m, axis)

def fromfunction(args, dimensions):
    return np.fromfunction(args, dimensions, dtype=int)

def ones(shape, typecode='l', savespace=0, dtype=None):
    """ones(shape, dtype=int) returns an array of the given
    dimensions which is initialized to all ones.
    """
    dtype = convtypecode(typecode, dtype)
    a = mu.empty(shape, dtype)
    a.fill(1)
    return a

def zeros(shape, typecode='l', savespace=0, dtype=None):
    """zeros(shape, dtype=int) returns an array of the given
    dimensions which is initialized to all zeros
    """
    dtype = convtypecode(typecode, dtype)
    return mu.zeros(shape, dtype)

def identity(n,typecode='l', dtype=None):
    """identity(n) returns the identity 2-d array of shape n x n.
    """
    dtype = convtypecode(typecode, dtype)
    return nn.identity(n, dtype)

def empty(shape, typecode='l', dtype=None):
    dtype = convtypecode(typecode, dtype)
    return mu.empty(shape, dtype)

def array(sequence, typecode=None, copy=1, savespace=0, dtype=None):
    dtype = convtypecode2(typecode, dtype)
    return mu.array(sequence, dtype, copy=copy)

def sarray(a, typecode=None, copy=False, dtype=None):
    dtype = convtypecode2(typecode, dtype)
    return mu.array(a, dtype, copy)

def asarray(a, typecode=None, dtype=None):
    dtype = convtypecode2(typecode, dtype)
    return mu.array(a, dtype, copy=0)

def nonzero(a):
    res = np.nonzero(a)
    if len(res) == 1:
        return res[0]
    else:
        raise ValueError("Input argument must be 1d")

def reshape(a, shape):
    return np.reshape(a, shape)

def arange(start, stop=None, step=1, typecode=None, dtype=None):
    dtype = convtypecode2(typecode, dtype)
    return mu.arange(start, stop, step, dtype)

def fromstring(string, typecode='l', count=-1, dtype=None):
    dtype = convtypecode(typecode, dtype)
    return mu.fromstring(string, dtype, count=count)

def ravel(m):
    return np.ravel(m)

def trace(a, offset=0, axis1=0, axis2=1):
    return np.trace(a, offset=0, axis1=0, axis2=1)

def indices(dimensions, typecode=None, dtype=None):
    dtype = convtypecode(typecode, dtype)
    return np.indices(dimensions, dtype)

def where(condition, x, y):
    return np.where(condition, x, y)

def cross_product(a, b, axis1=-1, axis2=-1):
    return np.cross(a, b, axis1, axis2)

def average(a, axis=0, weights=None, returned=False):
    return np.average(a, axis, weights, returned)
